/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (c) 2023, Linaro Ltd.
 */

#ifndef _DT_BINDINGS_CLK_QCOM_LPASSCC_SC8280XP_H
#define _DT_BINDINGS_CLK_QCOM_LPASSCC_SC8280XP_H

/* LPASS AUDIO CC CSR */
#define LPASS_AUDIO_SWR_RX_CGCR				0
#define LPASS_AUDIO_SWR_WSA_CGCR			1
#define LPASS_AUDIO_SWR_WSA2_CGCR			2

/* LPASS TCSR */
#define LPASS_AUDIO_SWR_TX_CGCR				0

#endif
