/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef __ASM_GENERIC_EXPORT_H
#define __ASM_GENERIC_EXPORT_H

/*
 * <asm/export.h> and <asm-generic/export.h> are deprecated.
 * Please include <linux/export.h> directly.
 */
#include <linux/export.h>

#endif
